unit decoder;

interface

uses types;

function  cmd_text (cmd_in: VOCABULARY):STRING;
function  txt      (txt_in: VOCABULARY):STRING;
function  decode_word (text_in: STRING):VOCABULARY;
procedure decode_double_words (var line: STRING);
procedure remove_the (var line: STRING);
procedure downcase (var text: STRING);

implementation

function cmd_text (cmd_in: VOCABULARY):STRING;
begin

  case cmd_in of
    again:      cmd_text := 'AGAIN';
    all:        cmd_text := 'ALL';
    and_:       cmd_text := 'AND';
    but:        cmd_text := 'BUT';
    close:      cmd_text := 'CLOSE';
    cut:        cmd_text := 'CUT';
    drop:       cmd_text := 'DROP';
    eat:        cmd_text := 'EAT';
    examine:    cmd_text := 'EXAMINE';
    from:       cmd_text := 'FROM';
    get:        cmd_text := 'GET';
    get_in_to:  cmd_text := 'GET IN TO';
    get_out_of: cmd_text := 'GET OUT OF';
    in_:        cmd_text := 'IN';
    inventory:  cmd_text := 'INVENTORY';
    it:         cmd_text := 'IT';
    lock:       cmd_text := 'LOCK';
    look:       cmd_text := 'LOOK';
    of_:        cmd_text := 'OF';
    off:        cmd_text := 'OFF';
    on:         cmd_text := 'ON';
    oops:       cmd_text := 'OOPS';
    open:       cmd_text := 'OPEN';
    out:        cmd_text := 'OUT';
    pick:       cmd_text := 'PICK';
    put:        cmd_text := 'PUT';
    quit:       cmd_text := 'QUIT';
    remove:     cmd_text := 'REMOVE';
    score:      cmd_text := 'SCORE';
    take:       cmd_text := 'TAKE';
    time:       cmd_text := 'TIME';
    to_:        cmd_text := 'TO';
    unlock:     cmd_text := 'UNLOCK';
    wait:       cmd_text := 'WAIT';
    wake:       cmd_text := 'WAKE';
    wear:       cmd_text := 'WEAR';
    with_:      cmd_text := 'WITH';
  end; (* case *)

end; (* cmd_text *)

function txt (txt_in: VOCABULARY):STRING;
begin

  case txt_in of
    couch:    txt := 'pilot''s couch';
    hatch:    txt := 'scooter''s hatch';
    office:   txt := 'director''s office';
    scooter:  txt := 'scooter';
    store:    txt := 'store room';
    sick_bay: txt := 'sick bay';
  end;

end; (* txt *)

function decode_word (text_in: STRING):VOCABULARY;

  function isCommand (command_in: VOCABULARY):BOOLEAN;
  begin
    isCommand := (text_in = cmd_text(command_in));
  end;

  function isTxt (txt_in: VOCABULARY):BOOLEAN;
  var cap_txt: STRING;
      element: INTEGER;
  begin

    cap_txt := txt(txt_in);

    for element := 1 to length (cap_txt) do
      cap_txt [element] := upcase (cap_txt [element]);

    isTxt := (text_in = cap_txt);

  end;

  function isWord (a: STRING):BOOLEAN;
  begin
    isWord := (text_in = a);
  end;

  function isWord2 (a, b: STRING):BOOLEAN;
  begin
    isWord2 := (text_in = a) or (text_in = b);
  end;

var command, code: VOCABULARY;
begin (* decode_word *)

  code := rubbish;

  for command := first_command to last_command do
    if isCommand(command) then
      code := command;

       if isWord ('EXCEPT') then code := but
  else if isWord ('I')      then code := inventory
  else if isWord ('L')      then code := look
  else if isWord ('Q')      then code := quit
  else if isWord ('Z')      then code := wait

  else if isWord2 ('BAR', 'BARS')   then code := bars
  else if isWord  ('COUCH')         then code := couch
  else if isWord2 ('HATCH', 'DOOR') then code := hatch
  else if isWord  ('FUSSBUDGET')    then code := fuss_budget

  else if isWord2 ('GO', 'WALK')                           then code := go
  else if isWord2 ('NORTH', 'N') or isWord2 ('FORE',  'F') then code := north
  else if isWord2 ('EAST',  'E')
       or isWord2 ('STARBOARD', 'SB')                      then code := east
  else if isWord2 ('SOUTH', 'S') or isWord2 ('AFT',  'A')  then code := south
  else if isWord2 ('WEST',  'W') or isWord2 ('PORT', 'P')  then code := west
  else if isWord2 ('UP',    'U')                           then code := up
  else if isWord2 ('DOWN',  'D')                           then code := down
  else if isWord  ('ENTER')                                then code := enter
  else if isWord  ('LEAVE') or isWord ('EXIT')             then code := leave
  else if isWord  ('STAND')                                then code := stand
  else if isWord  ('SIT')                                  then code := sit

  else if isWord ('BAY')    then code := sick_bay
  else if isWord ('OFFICE') then code := office
  else if isWord ('ROOM')   then code := room_
  else if isTxt  (scooter)  then code := scooter
  else if isWord ('STORE')  then code := store

  else if isWord  ('ADAPTER')              then code := plug
  else if isWord  ('BRACELET')             then code := bracelet
  else if isWord  ('CABINET')              then code := cabinet
  else if isWord  ('CAGE')                 then code := cage
  else if isWord2 ('CASE', 'FRUIT')        then code := case_of_fruit
  else if isWord2 ('CHRONOMETER', 'WATCH') then code := watch
  else if isWord  ('COMPUTER')             then code := computer
  else if isWord  ('DESK')                 then code := desk
  else if isWord  ('ISOTOPES')             then code := isotopes
  else if isWord  ('KEY')                  then code := key
  else if isWord2 ('PASTE', 'TUBE')        then code := paste
  else if isWord  ('PLUG')                 then code := plug
  else if isWord  ('SAW')                  then code := hacksaw
  else if isWord  ('TOOLBOX')              then code := toolbox
  else if isWord  ('UNIFORM')              then code := uniform

  else if isWord ('') then code := return;

  decode_word := code;

end; (* decode_word *)

procedure decode_double_words (var line: STRING);
const array_size1 = 22;
var list:   ARRAY [1..array_size1] of STRING;
    offset: ARRAY [1..array_size1] of INTEGER;
    length: ARRAY [1..array_size1] of INTEGER;
    i: INTEGER;
begin

  list[ 1] := 'PILOTS COUCH';         offset[ 1] := 0; length[ 1] :=  7;
  list[ 2] := 'PILOT''S COUCH';       offset[ 2] := 0; length[ 2] :=  8;
  list[ 3] := 'SCOOTERS HATCH';       offset[ 3] := 0; length[ 3] :=  9;
  list[ 4] := 'SCOOTER''S HATCH';     offset[ 4] := 0; length[ 4] := 10;
  list[ 5] := 'DIRECTORS OFFICE';     offset[ 5] := 0; length[ 5] := 10;
  list[ 6] := 'DIRECTOR''S OFFICE';   offset[ 6] := 0; length[ 6] := 11;
  list[ 7] := 'SICK BAY';             offset[ 7] := 0; length[ 7] :=  5;
  list[ 8] := 'STORE ROOM';           offset[ 8] := 5; length[ 8] :=  5;
  list[ 9] := 'ADAPTER PLUG';         offset[ 9] := 0; length[ 9] :=  8;
  list[10] := 'CASE OF FRUIT';        offset[10] := 0; length[10] :=  8;
  list[11] := 'FILE CABINET';         offset[11] := 0; length[11] :=  5;
  list[12] := 'PORTABLE COMPUTER';    offset[12] := 0; length[12] :=  9;
  list[13] := 'RADIOACTIVE ISOTOPES'; offset[13] := 0; length[13] := 12;
  list[14] := 'BOX OF ISOTOPES';      offset[14] := 0; length[14] :=  7;
  list[15] := 'PATROL UNIFORM';       offset[15] := 0; length[15] :=  7;
  list[16] := 'IDENTITY BRACELET';    offset[16] := 0; length[16] :=  9;
  list[17] := 'HACKSAW';              offset[17] := 0; length[17] :=  4;
  list[18] := 'NUTRIENT PASTE';       offset[18] := 0; length[18] :=  9;
  list[19] := 'TUBE OF PASTE';        offset[19] := 0; length[19] :=  8;
  list[20] := 'ADAPTER PLUG';         offset[20] := 7; length[20] :=  5;
  list[21] := 'SIT DOWN';             offset[21] := 3; length[21] :=  5;
  list[22] := 'STAND UP';             offset[22] := 5; length[22] :=  3;

  for i := 1 to array_size1 do
    if pos (list[i], line) <> 0 then
      delete (line, pos (list[i], line) + offset[i], length[i]);

end; (* decode_double_words *)

procedure remove_the (var line: STRING);
const array_size2 = 25;
var list: ARRAY [1..array_size2] of STRING;
    i: INTEGER;
begin

  list[ 1] := 'COUCH';
  list[ 2] := 'DOOR';
  list[ 3] := 'HATCH';
  list[ 4] := 'BAY';
  list[ 5] := 'OFFICE';
  list[ 6] := 'SCOOTER';
  list[ 7] := 'STORE';
  list[ 8] := 'FUSSBUDGET';
  list[ 9] := 'ADAPTER';
  list[10] := 'BAR';
  list[11] := 'BRACELET';
  list[12] := 'CABINET';
  list[13] := 'CAGE';
  list[14] := 'CASE';
  list[15] := 'COMPUTER';
  list[16] := 'DESK';
  list[17] := 'FRUIT';
  list[18] := 'ISOTOPES';
  list[19] := 'KEY';
  list[20] := 'PASTE';
  list[21] := 'PLUG';
  list[22] := 'SAW';
  list[23] := 'TOOLBOX';
  list[24] := 'TUBE';
  list[25] := 'UNIFORM';

  for i := 1 to array_size2 do
    if pos ('THE '+list[i], line) <> 0 then
      delete (line, pos ('THE '+list[i], line), 4);

end; (* remove_the *)

procedure downcase (var text: STRING);
var element: INTEGER;
begin
  for element := 1 to length (text) do
    if text[element] in ['A'..'Z'] then
      text[element] := chr (ord(text[element]) + 32);
end;

end.