unit map;

interface

uses types, obj_char, actors,

 decoder;

type MAP_OBJECT = OBJECT (CHARACTERISTIC_OBJECT)
       procedure init_map;
       procedure describe_room    (description: DESCRIBE_TYPE);
       procedure core_move        (next_room: VOCABULARY);
       procedure enter_leave_proc (word_in, room_in: VOCABULARY);
       procedure move_proc        (word_in: VOCABULARY);
       procedure sit_in        (special_object_in: VOCABULARY);
       procedure stand_up_from (special_object_in: VOCABULARY);
       procedure sit_proc;
       procedure get_in_to_proc;
       procedure get_out_of_proc;
       procedure stand_proc;
       procedure open_hatch;
       procedure close_hatch;
       procedure look_proc        (description: DESCRIBE_TYPE);
     end;

implementation

procedure MAP_OBJECT.init_map;
begin

  room [sick_bay].name  := 'Sick Bay';
  room [sick_bay].line1 := '   Like the rest of the station, the medical facility is deserted. This is odd,';
  room [sick_bay].line2 := 'for regulations require that the sick bay be staffed around the clock, ten';
  room [sick_bay].line3 := 'chrons a day! The only exit is aft. ';
  room [sick_bay].north := wall;
  room [sick_bay].east  := wall;
  room [sick_bay].south := hall_n;
  room [sick_bay].west  := wall;
  room [sick_bay].up    := wall;
  room [sick_bay].down  := wall;
  room [sick_bay].enter := wall;
  room [sick_bay].leave := hall_n;
  room [sick_bay].new := true;

  room [hall_l].name := 'Main Hallway, Sector L';
  room [hall_l].line1 := '   This is the stations main corridor, which continues to port and starboard.';
  room [hall_l].line2 := 'Rooms lie fore and aft. ';
  room [hall_l].line3 := '';
  room [hall_l].north := store;
  room [hall_l].east  := hall_m;
  room [hall_l].south := office;
  room [hall_l].west  := unimplemented;
  room [hall_l].up    := wall;
  room [hall_l].down  := wall;
  room [hall_l].enter := wall;
  room [hall_l].leave := wall;
  room [hall_l].new := true;

  room [hall_m].name := 'Main Hallway, Sector M';
  room [hall_m].line1 := '   This is the stations main corridor, which continues to port and starboard. A';
  room [hall_m].line2 := 'tube leads up toward the docking port. ';
  room [hall_m].line3 := '';
  room [hall_m].north := wall;
  room [hall_m].east  := hall_n;
  room [hall_m].south := wall;
  room [hall_m].west  := hall_l;
  room [hall_m].up    := docking_port;
  room [hall_m].down  := wall;
  room [hall_m].enter := wall;
  room [hall_m].leave := wall;
  room [hall_m].new := true;

  room [hall_n].name := 'Main Hallway, Sector N';
  room [hall_n].line1 := '   This is the stations main corridor, which continues to port and starboard. A';
  room [hall_n].line2 := 'room lies to fore, and a tube leads down to the next level of the station. ';
  room [hall_n].line3 := '';
  room [hall_n].north := sick_bay;
  room [hall_n].east  := unimplemented;
  room [hall_n].south := wall;
  room [hall_n].west  := hall_m;
  room [hall_n].up    := wall;
  room [hall_n].down  := unimplemented;
  room [hall_n].enter := sick_bay;
  room [hall_n].leave := wall;
  room [hall_n].new := true;

  room [office].name := 'Director''s Office';
  room [office].line1 := '   This is the office of the station manager. It is simply furnished, with a';
  room [office].line2 := 'file cabinet and a desk. There''s an electrical outlet on one wall. ';
  room [office].line3 := '';
  room [office].north := hall_l;
  room [office].east  := wall;
  room [office].south := wall;
  room [office].west  := wall;
  room [office].up    := wall;
  room [office].down  := wall;
  room [office].enter := wall;
  room [office].leave := hall_l;
  room [office].new := true;

  room [docking_port].name := 'Docking Port';
  room [docking_port].line1 := '   This is the huge docking port of the space station. Only one ship is here at';
  room [docking_port].line2 := 'the moment, a one-man scooter, so the docking port seems unusually empty. A';
  room [docking_port].line3 := 'tube leads down toward the heart of the station. ';
  room [docking_port].north := wall;
  room [docking_port].east  := wall;
  room [docking_port].south := wall;
  room [docking_port].west  := wall;
  room [docking_port].up    := wall;
  room [docking_port].down  := hall_m;
  room [docking_port].enter := scooter;
  room [docking_port].leave := wall;
  room [docking_port].new := true;

  room [scooter].name := 'Scooter';
  room [scooter].line1 := '   You are in the cramped one-man space scooter. Through the viewport, you see';
  room [scooter].line2 := 'the docking port of the Space Station BG-12. ';
  room [scooter].line3 := '';
  room [scooter].north := wall;
  room [scooter].east  := wall;
  room [scooter].south := wall;
  room [scooter].west  := wall;
  room [scooter].up    := wall;
  room [scooter].down  := wall;
  room [scooter].enter := wall;
  room [scooter].leave := closed_hatch;
  room [scooter].new := false;

  room [store].name := 'Store Room';
  room [store].line1 := '   This is a small storage room of some sort. A hallway lies aft.';
  room [store].line2 := '';
  room [store].line3 := '';
  room [store].north := wall;
  room [store].east  := wall;
  room [store].south := hall_l;
  room [store].west  := wall;
  room [store].up    := wall;
  room [store].down  := wall;
  room [store].enter := wall;
  room [store].leave := hall_l;
  room [store].new := true;

  when := 1;
  current_room := scooter;

end; (* init_map *)

procedure MAP_OBJECT.describe_room (description: DESCRIBE_TYPE);
begin

  if (description = verbose) then
  begin
    spool_text (room[current_room].line1);
    spool_text (room[current_room].line2);
    spool_text (room[current_room].line3);
    room [current_room].new := false;
  end;

  if (current_room in [scooter, docking_port]) then
    if (room[scooter].leave = closed_hatch)
    or (room[docking_port].enter = closed_hatch) then
      observe (hatch_close)
    else
      observe (hatch_open);

  if (current_room = sick_bay) then
    if (cage_bars <> two_cut) then
      if fussbudget.awake then
      begin
        spool_text ('   At the far end of the room, inside a steel cage is a Thoosian Fussbudget,');
        spool_text ('it seems to be awake.');
      end
      else
      begin
        spool_text ('   At the far end of the room, inside a steel cage is a Thoosian Fussbudget,');
        spool_text ('apparently asleep.');
      end
    else
      spool_text ('   At the far end of the room is an empty steel cage.');

end; (* describe_room *)

procedure MAP_OBJECT.core_move (next_room: VOCABULARY);
var element: VOCABULARY;
begin

  if (next_room = docking_port) and is_sitting then
    error (get_up)
  else
    case next_room of
      closed_hatch:  error (hatch_close);
      unimplemented: error (unimplemented_direction);
      wall:          error (no_exit);
    end; (* case *)

  if isRoom (next_room) and (next_room <> current_room) and not is_sitting then
  begin

    current_room := next_room;

    if room[current_room].new then
      look_proc (verbose)
    else
      look_proc (brief);

    advance_time (22);

  end; (* if *)

end; (* core_move *)

procedure MAP_OBJECT.enter_leave_proc (word_in, room_in: VOCABULARY);
var next_room: VOCABULARY;
begin

  if room_in in [office, store, sick_bay] then
  begin
    if (word_in = enter) then
    begin
      if ((room_in = office)   and (current_room = hall_l))
      or ((room_in = store)    and (current_room = hall_l))
      or ((room_in = sick_bay) and (current_room = hall_n)) then
        next_room := room_in
      else if (current_room = room_in) then
        error_already_in (room_in)
      else
        error (which_direction)
    end

    else if (word_in = leave) then
    begin
      if ((room_in = office)   and (current_room = office))
      or ((room_in = store)    and (current_room = store))
      or ((room_in = sick_bay) and (current_room = sick_bay)) then
        next_room := room[current_room].leave
      else
        error_not_in (room_in);
    end
  end

  else if (room_in = scooter) then
  begin
    if (word_in = enter) then
      if (current_room = docking_port) then
        next_room := scooter
      else if (current_room = scooter) then
        error_already_in (current_room)
      else
        error_you_cant_see (scooter)
    else if (word_in = leave) then
      if (current_room = scooter) then
        next_room := room[current_room].leave
      else if (current_room = docking_port) then
        error_not_in (scooter)
      else
        error_you_cant_see (scooter)
  end;

  core_move (next_room);

end; (* enter_leave_proc *)

procedure MAP_OBJECT.move_proc (word_in: VOCABULARY);
var next_room: VOCABULARY;
begin

  case word_in of
    north: next_room := room[current_room].north;
    east:  next_room := room[current_room].east;
    south: next_room := room[current_room].south;
    west:  next_room := room[current_room].west;
    up:    next_room := room[current_room].up;
    down:  next_room := room[current_room].down;
    enter: next_room := room[current_room].enter;
    leave: next_room := room[current_room].leave;
  end; (* case *)

  if (current_room = scooter) and (word_in = enter) then
    sit_proc
  else if (next_room = wall) and (word_in in [enter, leave]) then
    error (which_direction)
  else
    core_move (next_room);

end; (* move_proc *)

procedure MAP_OBJECT.sit_in (special_object_in: VOCABULARY);
begin

  if (special_object_in = couch) then
    if is_sitting then
      error_already_in (special_object_in)
    else
    begin
      is_sitting := true;
      spool_time ('You are now in the '+txt(special_object_in)+'.', 7);
    end;

end; (* sit_in *)

procedure MAP_OBJECT.stand_up_from (special_object_in: VOCABULARY);
begin

  if (special_object_in = couch) then
    if is_sitting then
    begin
      is_sitting := false;
      spool_time ('You get out of the '+txt(special_object_in)+'.', 7);
    end
    else
      error_not_in (special_object_in);

end; (* stand_up_from *)

procedure MAP_OBJECT.sit_proc;
begin
  if (current_room <> scooter) then
  begin
    spool_text ('You recline on the floor for a bit, and stand up again.');
    advance_time (38);
  end
  else
    sit_in (couch);
end; (* sit_proc *)

procedure MAP_OBJECT.stand_proc;
begin

  if is_sitting then
  begin
    is_sitting := false;
    spool_time ('You are standing again.', 7);
  end
  else
    error (already_standing);

end; (* stand_proc *)

procedure MAP_OBJECT.get_in_to_proc;
begin

  if phrase1 (get_in_to) then
  begin
    if (current_room = scooter) then
    begin
      observe_the (couch);
      sit_in (couch);
    end
    else
      error_what (get_in_to);
  end
  else if phrase2 (get_in_to, couch) and (current_room = scooter) then
    sit_in (couch)
  else if (word[1].current = get_in_to)
  and (word[2].current in [scooter, office, store, sick_bay])
  and (word[3].current = return) then
    enter_leave_proc (enter, word[2].current)
  else if phrase2 (get_in_to, any_object) then
    spool_vowel ('You can''t get into ',word[2].current,'!')
  else if phrase2 (get_in_to, any_special_object) then
    spool_special_vowel ('You can''t get into ',word[2].current,'!')
  else
    error (dont_understand);

end; (* get_in_proc *)

procedure MAP_OBJECT.get_out_of_proc;
begin

  if phrase1 (get_out_of) then
  begin
    if (current_room = scooter) then
    begin
      observe_the (couch);
      stand_up_from (couch);
    end
    else
      error_what (get_out_of);
  end
  else if phrase2 (get_out_of, couch) and (current_room = scooter) then
    stand_up_from (couch)
  else if (word[1].current = get_out_of)
  and (word[2].current in [scooter, office, store, sick_bay])
  and (word[3].current = return) then
    enter_leave_proc (leave, scooter)
  else
    error (dont_understand);

end; (* get_out_of_proc *)

procedure MAP_OBJECT.open_hatch;
begin

  if (current_room <> scooter) and (current_room <> docking_port) then
    error_you_cant_see (hatch)
  else if is_sitting then
    error_you_cant_reach (hatch)
  else
  begin

    if (current_room = scooter) then
    begin

      if (clock >= 4550) then

        if (room [scooter].leave = closed_hatch) then
        begin
          room [scooter].leave := docking_port;
          action (hatch_open);
          advance_time (7);
        end
        else
          error (already_open)
      else
        error (deep_space)
    end; (* if *)

    if (current_room = docking_port) then

      if (room [docking_port].enter = closed_hatch) then
      begin
        room [docking_port].enter := scooter;
        action (hatch_open);
        advance_time (7);
      end
      else
        error (already_open)

  end; (* if *)

end; (* open_hatch *)

procedure MAP_OBJECT.close_hatch;
begin

  if (current_room <> scooter) and (current_room <> docking_port) then
    error_you_cant_see (hatch)
  else if is_sitting then
    error_you_cant_reach (hatch)
  else
  begin

    if (current_room = scooter) then

      if (room [scooter].leave = docking_port) then
      begin
        room [scooter].leave := closed_hatch;
        action (hatch_close);
        advance_time (7);
      end
      else
        error (already_closed);

    if (current_room = docking_port) then

      if (room [docking_port].enter = scooter) then
      begin
        room [docking_port].enter := closed_hatch;
        action (hatch_close);
        advance_time (7);
      end
      else
        error (already_closed);

  end; (* if *)

end; (* close_hatch *)

procedure MAP_OBJECT.look_proc (description: DESCRIBE_TYPE);
var element: VOCABULARY;
begin

  write (room[current_room].name);
  if is_sitting then write (', in the pilot''s couch');
  spool;
  describe_room (description);
  list_objects (current_room);

  for element := object_first to object_last do
    if isInRoom(element) and (list[element].inside = opened) then
    begin
      current_line := 'The '+list[element].name+' contains ';
      list_objects_sub (element);
    end;

  advance_time (16);

end; (* look_proc *)

end.