unit obj;

interface

uses types, general;

type LIST_OBJECT = OBJECT (GENERAL_OBJECT)
       list: array [bracelet..watch] of OBJECT_;
       last_object: VOCABULARY;
       when: INTEGER;
       current_line: STRING;
       function  isWearable (object_in: VOCABULARY):BOOLEAN;
       function  isHeavy    (object_in: VOCABULARY):BOOLEAN;
       function  isCarrying (object_in: VOCABULARY):BOOLEAN;
       function  isWearing  (object_in: VOCABULARY):BOOLEAN;
       function  isInRoom   (object_in: VOCABULARY):BOOLEAN;
       function  isHeld     (object_in: VOCABULARY):BOOLEAN;
       function  isHere     (object_in: VOCABULARY):BOOLEAN;
       function  number_of_objects (location: VOCABULARY):INTEGER;
       function  no_objects        (location: VOCABULARY):BOOLEAN;
       function  one_object        (location: VOCABULARY):BOOLEAN;
       function  sole_object       (location: VOCABULARY):VOCABULARY;
       function  no_objects_held:BOOLEAN;
       function  one_object_held:BOOLEAN;
       function  object_held    :VOCABULARY;
       procedure spool_name (text_start: STRING; object_in: VOCABULARY; text_end: STRING);
       procedure spool_vowel (text_start: STRING; object_in: VOCABULARY; text_end: STRING);
       procedure init_objects;
       procedure list_objects_sub (place: VOCABULARY);
       procedure list_objects     (place: VOCABULARY);
       procedure inventory_proc;
       procedure examine_router;
       procedure examine_proc (object_in: VOCABULARY);
       procedure inc_when;
     end;

implementation

function LIST_OBJECT.isWearable (object_in: VOCABULARY):BOOLEAN;
begin
  isWearable := list[object_in].characteristic = wearable;
end;

function LIST_OBJECT.isHeavy (object_in: VOCABULARY):BOOLEAN;
begin
  isHeavy := list[object_in].characteristic = heavy;
end;

function LIST_OBJECT.isCarrying (object_in: VOCABULARY):BOOLEAN;
begin
  isCarrying := list[object_in].place = carrying;
end;

function LIST_OBJECT.isWearing (object_in: VOCABULARY):BOOLEAN;
begin
  isWearing := list[object_in].place = wearing;
end;

function LIST_OBJECT.isInRoom (object_in: VOCABULARY):BOOLEAN;
begin
  isInRoom := list[object_in].place = current_room;
end;

function LIST_OBJECT.isHeld (object_in: VOCABULARY):BOOLEAN;
begin
  if (list[object_in].place = carrying)
  or (list[object_in].place = wearing) then
    isHeld := TRUE
  else
    isHeld := FALSE;
end;

function LIST_OBJECT.isHere (object_in: VOCABULARY):BOOLEAN;
begin
  if (list[object_in].place = carrying)
  or (list[object_in].place = wearing)
  or (list[object_in].place = current_room) then
    isHere := TRUE
  else
    isHere := FALSE;
end;

function LIST_OBJECT.number_of_objects (location: VOCABULARY):INTEGER;
var objects: INTEGER;
    element: VOCABULARY;
begin

  objects := 0;

  for element := object_first to object_last do
    if list[element].place = location then
      objects := objects + 1;

  number_of_objects := objects;

end; (* number_of_objects *)

function L                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                gin
      objects := objects + 1;
      if objects = 1 then
        single_object := element;
    end;

  sole_object := single_object;

end; (* sole_object *)

function LIST_OBJECT.no_objects_held:BOOLEAN;
begin
  no_objects_held := no_objects (carrying) and no_objects (wearing);
end;

function LIST_OBJECT.one_object_held:BOOLEAN;
begin
  one_object_held := number_of_objects (carrying) +
                     number_of_objects (wearing) = 1;
end;

function LIST_OBJECT.object_held:VOCABULARY;
var objects_held: INTEGER;
    element, single_object: VOCABULARY;
begin

  objects_held := 0;

  for element := object_first to object_last do
    if isHeld (element) then
    begin
      objects_held := objects_held + 1;
      if objects_held = 1 then
        single_object := element;
    end;

  object_held := single_object;

end; (* object_held *)

procedure LIST_OBJECT.spool_name (text_start: STRING; object_in: VOCABULARY; text_end: STRING);
begin
  spool_text (text_start + list[object_in].name + text_end)
end;

procedure LIST_OBJECT.spool_vowel (text_start: STRING; object_in: VOCABULARY; text_end: STRING);
begin
  case list[object_in].name[1] of
    'a', 'e', 'i', 'o', 'u': spool_text (text_start + 'an' + list[object_in].name + text_end);
  else
    spool_text (text_start + 'a ' + list[object_in].name + text_end);
  end;
end;

procedure LIST_OBJECT.init_objects;
begin

  list [bracelet].name := 'identity bracelet';
  list [bracelet].place := toolbox;
  list [bracelet].characteristic := wearable;
  list [bracelet].lock := none;
  list [bracelet].inside := none;
  list [bracelet].new := false;

  list [cabinet].name := 'file cabinet';
  list [cabinet].place := office;
  list [cabinet].characteristic := heavy;
  list [cabinet].lock := locked;
  list [cabinet].inside := closed;
  list [cabinet].new := false;

  list [cage].name := 'cage';
  list [cage].place := sick_bay;
  list [cage].characteristic := heavy;
  list [cage].lock := locked;
  list [cage].inside := closed;
  list [cage].new := true;
  list [cage].line[1] := 'Basically the cage is just a collection of steel bars welded together.';
  list [cage].line[2] := '';
  list [cage].line[3] := '';

  list [case_of_fruit].name := 'case of fruit';
  list [case_of_fruit].place := store;
  list [case_of_fruit].characteristic := edible;
  list [case_of_fruit].lock := none;
  list [case_of_fruit].inside := none;
  list [case_of_fruit].new := false;
  list [case_of_fruit].eat_line[1] := 'Mmm...one of your favorites...mondoweed fruits.';
  list [case_of_fruit].eat_line[2] := 'You empty the entire case, and your hunger dissipates.';

  list [computer].name := 'portable computer';
  list [computer].place := office;
  list [computer].characteristic := none;
  list [computer].lock := none;
  list [computer].inside := none;
  list [computer].new := false;
  list [computer].line[1] := 'The portable computer has a screen, a keyboard, and a power cord.';
  list [computer].line[2] := 'The screen is blank.';
  list [computer].line[3] := '';

  list [desk].name := 'desk';
  list [desk].place := office;
  list [desk].characteristic := heavy;
  list [desk].lock := none;
  list [desk].inside := none;
  list [desk].new := false;

  list [hacksaw].name := 'hacksaw';
  list [hacksaw].place := toolbox;
  list [hacksaw].characteristic := cuts;
  list [hacksaw].lock := none;
  list [hacksaw].inside := none;
  list [hacksaw].new := false;

  list [isotopes].name := 'box of radioactive isotopes';
  list [isotopes].place := carrying;
  list [isotopes].characteristic := none;
  list [isotopes].lock := none;
  list [isotopes].inside := none;
  list [isotopes].new := false;

  list [key].name := 'key';
  list [key].place := hall_n;
  list [key].characteristic := lock_unlock;
  list [key].lock := none;
  list [key].inside := none;
  list [key].new := true;
  list [key].description := '   Lying at the base of the wall is a key.';
  list [key].line[1] := 'The key looks like one of those new all-purpose keys that molds itself to any';
  list [key].line[2] := 'lock. A typical example of Garfoogian ingenuity.';
  list [key].line[3] := '';

  list [paste].name := 'tube of nutrient paste';
  list [paste].place := toolbox;
  list [paste].characteristic := edible;
  list [paste].lock := none;
  list [paste].inside := none;
  list [paste].new := false;
  list [paste].eat_line[1] := 'Mmm...one of your favorites...anchovies and peanut butter.';
  list [paste].eat_line[2] := 'You empty the entire tube, and your hunger dissipates.';

  list [plug].name := 'adapter plug';
  list [plug].place := store;
  list [plug].characteristic := none;
  list [plug].lock := none;
  list [plug].inside := none;
  list [plug].new := false;
  list [plug].line[1] := 'The adapter seems designed to fit nine-prong plugs into five-prong outlets.';
  list [plug].line[2] := '';

  list [toolbox].name := 'toolbox';
  list [toolbox].place := scooter;
  list [toolbox].characteristic := none;
  list [toolbox].lock := locked;
  list [toolbox].inside := closed;
  list [toolbox].new := false;

  list [uniform].name := 'patrol uniform';
  list [uniform].place := wearing;
  list [uniform].characteristic := wearable;
  list [uniform].lock := none;
  list [uniform].inside := closed;
  list [uniform].new := false;
  list [uniform].line[1] := 'It is a standard-issue one-pocket Stellar Patrol uniform, a miracle of modern';
  list [uniform].line[2] := 'technology. It will keep its owner warm in cold climates and cool in warm';
  list [uniform].line[3] := 'locales. It provides protection against mild radiation, repels all insects,';
  list [uniform].line[4] := 'absorbs sweat, promotes healthy skin tone, and on top of everything else, it';
  list [uniform].line[5] := 'is super-comfy.';

  list [watch].name := 'chronometer';
  list [watch].place := wearing;
  list [watch].characteristic := wearable;
  list [watch].lock := none;
  list [watch].inside := none;
  list [watch].new := false;

end; (* init_objects *)

procedure LIST_OBJECT.list_objects_sub (place: VOCABULARY);

var object_number, current_object: integer;
    element: vocabulary;

begin (* list_objects_sub *)

  object_number := 0; current_object := 0;

  for element := object_first to object_last do
    if (list [element].place = place) and not list [element].new then
      object_number := object_number + 1;

  if (object_number = 0) and (place = carrying) then
    current_line := 'You are empty-handed.'
  else if (object_number > 0) then
  begin

    if (place = carrying) then
      current_line := 'You are carrying '
    else if (place = wearing) then
      current_line := current_line + ' You are wearing '
    else if (list [place].inside <> opened) then
      current_line := '   You can see ';

    for element := object_first to object_last do

      if (list [element].place = place) and not list [element].new then
      begin

        current_object := current_object + 1;

        case list [element].name[1] of
          'a', 'e', 'i', 'o', 'u': current_line := current_line + 'an ';
        else
          current_line := current_line + 'a ';
        end; (* case *)

        current_line := current_line + list [element].name;

        if (current_object <= object_number - 2) then
          current_line := current_line + ', ';
        if (current_object = object_number - 1) then
          current_line := current_line + ' and ';

        send_line (current_line);

      end; (* if *)

    if (place = carrying) or (place = wearing)
    or (list [place].inside = opened) then
      current_line := current_line + '.'
    else
      current_line := current_line + ' here.';

  end; (* if *)

  if (list [place].inside = opened) then
    if (object_number = 0) then
      spool_name ('The ',place,' is empty.')
    else
      spool_text (current_line);

end; (* list_objects_sub *)

procedure LIST_OBJECT.list_objects (place: vocabulary);
var element: VOCABULARY;
begin

  current_line := '';

  list_objects_sub (place);

  if (place = carrying) then
    list_objects_sub (wearing);

  spool_text (current_line);

  for element := object_first to object_last do
    if (list [element].place = place) and list [element].new then
      spool_text (list [element].description);

end; (* list_objects *)

procedure LIST_OBJECT.inventory_proc;
var element: VOCABULARY;
begin

  list_objects (carrying);

  for element := object_first to object_last do
    if isHeld (element) and (list [element].inside = opened) then
    begin
      current_line := 'The '+list[element].name+' contains ';
      list_objects_sub (element);
    end;

  advance_time (18);

end; (* inventory_proc *)

procedure LIST_OBJECT.examine_router;
begin

  if phrase1 (examine) then
    examine_proc (return)
  else if phrase2 (examine, any_object)
  or phrase2 (examine, hatch) or phrase2 (examine, it) then
    examine_proc (word[2].current)
  else if phrase2 (examine, all) then
    spool_text ('[You can''t use multiple direct objects with "examine".]')
  else
    spool_text ('I don''t understand that sentence.');

end; (* examine_router *)

procedure LIST_OBJECT.examine_proc (object_in: VOCABULARY);
var element: INTEGER;
begin

  if (object_in = it) then
    if (when = 2) then
      object_in := last_object
    else
      object_in := return;

  if (object_in = return) then
  begin
    spool_text ('[Examine what?]');
    repeatCommand := examine;
  end
  else if (object_in = hatch) then
  begin
    if (current_room in [scooter, docking_port]) then
      if (room[docking_port].enter = closed_hatch)
      or (room[scooter].leave = closed_hatch) then
        spool_text ('It seems the scooter''s hatch is closed.')
      else
        spool_text ('It seems the scooter''s hatch is open.')
    else
      spool_text ('You can''t see any hatch here!');

    advance_time (32);
  end
  else if isHere (object_in) then
  begin
    if (list[object_in].lock = locked) then
      spool_name ('The ',object_in,' is locked.')
    else if (list[object_in].lock = unlocked)
    and (list[object_in].inside = closed) then
      spool_name ('The ',object_in,' is unlocked.')
    else if (list[object_in].inside = opened) then
    begin
      current_line := 'The '+list[object_in].name+' contains ';
      list_objects_sub (object_in);
    end;

    if (object_in = watch) then
      if isWearing (object_in) then
      begin
        spool_text ('It is a standard wrist chronometer with a digital display. Current Galactic');
        write      ('Standard Time (adjusted to your local day-cycle) is ',clock,'. The back is engraved'); spool;
        spool_text ('with the message "Good luck in the Patrol! Love, Mom and Dad."');
      end
      else
      begin
        spool_text ('It is a standard wrist chronometer with a digital display. Current Galactic');
        spool_text ('Standard Time is impossible to determine, since you''ve removed your');
        spool_text ('chronometer. The back is engraved with the message "Good luck in the Patrol!');
        spool_text ('Love, Mom and Dad."');
      end
    else
    begin
      element := 1;

      repeat
        spool_text (list [object_in].line[element]);
        element := element + 1;
      until (list [object_in].line[element-1] = '') or (element > MAX_LINE);
    end;

    advance_time (32);
  end
  else if (object_in <> it) and (when <> 2) then
    spool_text ('You can''t see that here!');

end; (* examine_proc *)

procedure LIST_OBJECT.inc_when;
begin
  when := when + 1; if when > 3 then when := 3;
end;

end.