unit parser;

interface

uses types, map, decoder, actors;

type PARSER_OBJECT = OBJECT (MAP_OBJECT)
       isOops, terminate: BOOLEAN;
       line, command, rubbish_word: STRING;
       rubbish_element: INTEGER;
       procedure init_parser;
       procedure get_line;
       procedure get_word (var word_out: WORD_TYPE);
       procedure quit_proc;
       procedure get_sentence;
       procedure process_sentence;
       procedure decode_double_command (command1, command2, new_command: VOCABULARY);
       procedure decode_triple_command (command1, command2, command3, new_command: VOCABULARY);
       procedure get_and_execute_sentence;
     end;

implementation

procedure PARSER_OBJECT.init_parser;
begin
  isOops    := FALSE;
  terminate := FALSE;
end;

procedure PARSER_OBJECT.get_line;
var element: integer;
begin

  write ('>'); readln (line);

  while pos ('..', line) <> 0 do
    delete (line, pos ('..', line), 1);

  for element := 1 to length (line) do
    line [element] := upcase (line [element]);

  decode_double_words (line);
  remove_the (line);

  inc_when;

end; (* get_line *)

procedure PARSER_OBJECT.get_word (var word_out: WORD_TYPE);
begin (* get_word *)

  while command [1] = ' ' do delete (command,1,1);

  if pos (' ', command) <> 0 then
  begin
    word_out.text := copy (command, 1, pos (' ', command) - 1);
    delete (command, 1, pos (' ', command));
  end
  else
  begin
    word_out.text := command;
    command := '';
  end; (* if *)

  word_out.current := decode_word (word_out.text);

end; (* get_word *)

procedure PARSER_OBJECT.quit_proc;

var answer: STRING;
    mistake: boolean;

begin (* quit_proc *)

  mistake := false;

  actors_quit_proc;
  score_proc;
  spool;

  write ('Do you wish to leave the game? > ');

  repeat

    if mistake then write ('Please answer YES or NO. > ');

    readln (answer);

    while answer [1] = ' ' do delete (answer, 1, 1);
    while answer [length (answer)] = ' ' do delete (answer, 1, length (answer));

    if (answer = 'YES') or (answer = 'yes') or (answer = 'Y') or (answer = 'y') then
    begin
      mistake := false; terminate := true;
    end
    else if (answer = 'NO') or (answer = 'no') or (answer = 'N') or (answer = 'n') then
    begin
      mistake := false; terminate := false;
    end
    else
      mistake := true;

  until not mistake;

end; (* quit_proc *)

procedure PARSER_OBJECT.get_sentence;
var element: INTEGER;
begin

  if pos ('.', line) <> 0 then
  begin
    command := copy (line, 1, pos ('.', line) - 1);
    delete (line, 1, pos ('.', line));
  end
  else if pos ('THEN ', line) <> 0 then
  begin
    command := copy (line, 1, pos ('THEN ', line) - 1);
    delete (line, 1, pos ('THEN ', line) + 4);
  end
  else
  begin
    command := line; line := '';
  end; (* if *)

  for element := 1 to MAX_WORD do
    get_word (word[element]);

end; (* get_sentence *)

procedure PARSER_OBJECT.decode_double_command (command1, command2, new_command: VOCABULARY);
var element: INTEGER;
begin
  if (word[1].current = command1) and (word[2].current = command2) then
  begin
    word[1].current := new_command;
    for element := 2 to MAX_WORD - 1 do
      word[element].current := word[element+1].current;
    word[MAX_WORD].current := return;
  end
end; (* decode_double_command *)

procedure PARSER_OBJECT.decode_triple_command (command1, command2, command3, new_command: VOCABULARY);
var element: INTEGER;
begin

  if  (word[1].current = command1) and (word[2].current = command2)
  and (word[3].current = command3) then
  begin
    word[1].current := new_command;
    for element := 2 to MAX_WORD - 1 do
      word[element].current := word[element+2].current;
    word[MAX_WORD].current := return;
  end;

end; (* decode_triple_command *)

procedure PARSER_OBJECT.process_sentence;
var element, element1: INTEGER;
    oops_word: VOCABULARY;
begin

  if phrase1 (again) then
    for element := 1 to MAX_WORD do
      word[element].current := word[element].last;

  for element := 1 to MAX_WORD do
    if (word[element].current = it) then
      if (when = 2) then
        word[element].current := last_object
      else
        word[element].current := return;

  decode_triple_command (get, in_, to_, get_in_to);
  decode_triple_command (get, out, of_, get_out_of);

  decode_double_command (examine, room_, look);
  decode_double_command (get, in_,       get_in_to);
  decode_double_command (enter, room_,   enter);
  decode_double_command (leave, room_,   leave);
  decode_double_command (pick, up,       take);
  decode_double_command (put, on,        wear);
  decode_double_command (take, off,      remove);

  if phrase1 (in_) then
    word[1].current := enter
  else if phrase1 (out) then
    word[1].current := leave
  else if (word[1].current = get) then
    word[1].current := take
  else if phrase2 (go, any_direction) then
    for element := 1 to MAX_WORD - 1 do
      word[element].current := word[element+1].current;

  if (repeatCommand in [lock, unlock]) and isObject (word[1].current) then
  begin
    for element := MAX_WORD downto 4 do
      word[element].current := word[element-3].current;
    word[3].current := with_;
    word[2].current := word[2].last;
    word[1].current := repeatCommand;
    repeatCommand := return;
  end
  else if (repeatCommand <> return) and isObject (word[1].current) then
  begin
    for element := MAX_WORD downto 2 do
      word[element].current := word[element-1].current;
    word[1].current := repeatCommand;
    repeatCommand := return;
  end
  else if (repeatCommand in [take, drop]) and (word[1].current = all) then
  begin
    word[1].current:=repeatCommand; word[2].current:=all; word[3].current:=return;
    repeatCommand := return;
  end;

  if phrase2 (oops, any_word) then
  begin
    oops_word := word[2].current;
    for element := 1 to MAX_WORD do
      word[element].current := word[element].last;
    word[rubbish_element].current := oops_word;
  end;

  element := 1;
  repeat
    if word[element].current = rubbish then
    begin
      rubbish_word := word[element].text;
      if word[element].text = 'THE' then
      begin
        for element1 := element to MAX_WORD - 1 do
          word[element1].current := word[element1+1].current;
        word[6].current := return;
        if element <> MAX_WORD then
          rubbish_word := word[element+1].text;
      end;
      downcase (rubbish_word);
      rubbish_element := element;
      isOops := TRUE;
    end;
    element:=element+1;
  until (element > MAX_WORD) or (word[element-1].current in [return, rubbish]);

  for element := 1 to MAX_WORD do
    word[element].last := word[element].current;

end; (* process_sentence *)

procedure PARSER_OBJECT.get_and_execute_sentence;
begin

  get_sentence;
  process_sentence;

  if isOops then
  begin
    spool_text ('[I don''t know the word "'+rubbish_word+'".]');
    isOops := FALSE;
  end
  else if word[1].current = return then
    spool_text ('[I beg your pardon?]')
  else if phrase1 (any_direction) then
    move_proc (word[1].current)
  else if phrase2 (open, hatch) then
    open_hatch
  else if phrase2 (close, hatch) then
    close_hatch
  else if (word[1].current in [enter, leave]) and
  (word[2].current in [scooter, sick_bay, store, office]) and
  (word[3].current = return) then
    enter_leave_proc (word[1].current, word[2].current)
  else if phrase2 (wake, fuss_budget) then
    wake_proc
  else if phrase2 (open, any_object) then
    open_proc (word[2].current)
  else if phrase2 (close, any_object) then
    close_proc (word[2].current)

  else if phrase2 (eat, any_object) then
    eat_proc (word[2].current)
  else if phrase4 (cut, bars, with_, any_object) then
    cut_bars_with_proc (word[4].current)
  else if phrase4 (cut, any_object, with_, any_object) then
    cut_with_proc (word[2].current, word[4].current)

  else if phrase2 (wear, any_object) then
    wear_proc (word[2].current)
  else if phrase2 (wear, all) then
    spool_text ('[You can''t use multiple direct objects with "wear".]')
  else if phrase2 (remove, any_object) then
    remove_proc (word[2].current)
  else if phrase2 (remove, all) then
    spool_text ('[You can''t use multiple direct objects with "remove".]')

  else if phrase2 (lock, any_object) then
    lock_proc (word[2].current)
  else if phrase4 (lock, any_object, with_, any_object) then
    lock_with_proc (word[2].current, word[4].current)
  else if phrase2 (unlock, any_object) then
    unlock_proc (word[2].current)
  else if phrase4 (unlock, any_object, with_, any_object) then
    unlock_with_proc (word[2].current, word[4].current)

  else
    case word[1].current of
      drop, put:  drop_router;
      examine:    examine_router;
      get_in_to:  get_in_to_proc;
      get_out_of: get_out_of_proc;
      take:       take_router;
    else
      begin
        if word[2].current = return then
          case word[1].current of
            inventory:    inventory_proc;
            look:         look_proc (verbose);
            quit:         quit_proc;
            time:         time_proc;
            wait:         wait_proc;
            score:        score_proc;
            sit:          sit_proc;
            stand:        stand_proc;
            wear:         wear_x_proc;
            remove:       remove_x_proc;
          else
            spool_text ('I don''t understand that sentence.')
          end (* case *)
        else
          spool_text ('I don''t understand that sentence.');
      end;
    end; (* case *)

  actors_proc;
  if line = '' then spool;

end; { get_and_execute_sentence }

end.