unit spooler;

interface

uses crt, types, global, decoder;

type SPOOLER_OBJECT = OBJECT (GLOBAL_OBJECT)
       line_count: INTEGER;
       repeatCommand: VOCABULARY;
       function  write_int (points_in: INTEGER):STRING;
       procedure init_spool;
       procedure init_repeatCommand;
       procedure spool;
       procedure spool_text (text_in:     STRING);
       procedure spool_special_name  (text_start: STRING; special_object_in: VOCABULARY; text_end: STRING);
       procedure spool_special_vowel (text_start: STRING; special_object_in: VOCABULARY; text_end: STRING);
       procedure send_line  (var line_in: STRING);
       procedure action  (action_in:  SPOOLER_TYPE);
       procedure observe (observe_in: SPOOLER_TYPE);
       procedure observe_the (object_in: VOCABULARY);
       procedure error   (error_in:   SPOOLER_TYPE);
       procedure error_already_in     (location_in: VOCABULARY);
       procedure error_not_in         (location_in: VOCABULARY);
       procedure error_you_cant_reach (object_in:   VOCABULARY);
       procedure error_you_cant_see   (object_in:   VOCABULARY);
       procedure error_i_dont_know (word_in: STRING);
       procedure error_what (cmd_in: VOCABULARY);
     end;

implementation

function SPOOLER_OBJECT.write_int (points_in: INTEGER):STRING;
var line_out: STRING;
begin
  line_out := '';
  if points_in in [10..80] then
    line_out := line_out + chr((points_in DIV 10)+48);
  line_out := line_out + chr((points_in MOD 10)+48);
  write_int := line_out;
end;

procedure SPOOLER_OBJECT.init_spool;
begin
  line_count := 0;
end; (* init_spool *)

procedure SPOOLER_OBJECT.init_repeatCommand;
begin
  repeatCommand := return;
end;

procedure SPOOLER_OBJECT.spool;
begin

  writeln;
  line_count := line_count + 1;

  if line_count > 22 then
  begin

    writeln ('**MORE**');
    repeat until keypressed;

    gotoxy (1,24); write ('        ');
    gotoxy (1,24);

    line_count := 0;

  end; (* if *)

end; (* spool *)

procedure SPOOLER_OBJECT.spool_text (text_in: STRING);
begin
  if text_in <> '' then
  begin
    write (text_in); spool;
  end;
end;

procedure SPOOLER_OBJECT.spool_special_name (text_start: STRING; special_object_in: VOCABULARY; text_end: STRING);
begin
  spool_text (text_start + txt(special_object_in) + text_end)
end;

procedure SPOOLER_OBJECT.spool_special_vowel (text_start: STRING; special_object_in: VOCABULARY; text_end: STRING);
var special_object_txt: STRING;
begin

  special_object_txt := txt(special_object_in);

  case special_object_txt[1] of
    'a', 'e', 'i', 'o', 'u': spool_text (text_start + 'an ' + special_object_txt + text_end);
  else
    spool_text (text_start + 'a ' + special_object_txt + text_end);
  end;

end;

procedure SPOOLER_OBJECT.send_line (var line_in: STRING);

var space_position: INTEGER;
    print_line: STRING;

begin (* send_line *)

  if length (line_in) > 79 then
  begin

    space_position := 79;

    repeat
      space_position := space_position - 1
    until (space_position = 0) or (line_in [space_position] = ' ');

    if space_position > 0 then
    begin

      print_line := line_in;
      delete (print_line, space_position, length (print_line) - (space_position - 1));
      delete (line_in, 1, space_position);

      spool_text (print_line);

    end; (* if *)

  end; (* if *)

end; (* send_line *)

procedure SPOOLER_OBJECT.action (action_in: SPOOLER_TYPE);
begin

  case action_in of
    hatch_close: spool_text ('Okay, the scooter''s hatch is now closed.');
    hatch_open:  spool_text ('The scooter''s hatch swings open.');
  end;

end;

procedure SPOOLER_OBJECT.observe (observe_in: SPOOLER_TYPE);
begin

  case observe_in of
    hatch_close: spool_text ('The scooter''s hatch is closed.');
    hatch_open:  spool_text ('The scooter''s hatch is open. ');
  end;

end; (* observe *)

procedure SPOOLER_OBJECT.observe_the (object_in: VOCABULARY);
begin
  if isSpecialObject (object_in) then
    spool_text ('[the '+txt(object_in)+']');
end;

procedure SPOOLER_OBJECT.error (error_in: SPOOLER_TYPE);
var text: STRING;
begin

  case error_in of
    already_closed:   text:='It''s already closed.';
    already_open:     text:='It''s already open.';
    already_standing: text:='You are already standing.';
    deep_space: text:='You can''t open the scooter''s hatch in deep space!';
    get_up: text:='You will have to get up from the pilot''s couch first.';
    dont_understand:  text:='I don''t understand that sentence.';
    no_exit:          text:='You can''t go that way.';
    hatch_close: text:='You will have to open the scooter''s hatch first.';
    unimplemented_direction: text:='Unimplemented.';
    which_direction: text:='Did you have any particular direction in mind?';
  end;

  spool_text (text);

end; (* error *)

procedure SPOOLER_OBJECT.error_already_in (location_in: VOCABULARY);
begin
  if location_in in [couch, office, scooter, store, sick_bay] then
    spool_text ('You are already in the '+txt(location_in)+'.');
end;

procedure SPOOLER_OBJECT.error_not_in (location_in: VOCABULARY);
begin
  if location_in in [couch, office, scooter, store, sick_bay] then
    spool_text ('You''re not in the '+txt(location_in)+'.');
end;

procedure SPOOLER_OBJECT.error_you_cant_reach (object_in: VOCABULARY);
begin

  case object_in of
    hatch: spool_text ('You can''t reach the scooter''s hatch from the pilot''s couch.')
  end;

end;

procedure SPOOLER_OBJECT.error_you_cant_see (object_in: VOCABULARY);
begin

  case object_in of
    hatch:   spool_text ('You can''t see a hatch here.');
    scooter: spool_text ('You can''t see any '+txt(scooter)+' here!');
  end;

end;

procedure SPOOLER_OBJECT.error_i_dont_know (word_in: STRING);
begin
  spool_text ('[I don''t know the word "'+word_in+'".]');
end;

procedure SPOOLER_OBJECT.error_what (cmd_in: VOCABULARY);
var downcase_text: STRING;
begin
  downcase_text := cmd_text(cmd_in);
  downcase (downcase_text);
  spool_text ('[what do you want to '+downcase_text+'?]');
  repeatCommand := cmd_in;
end;

end.

