unit types;

interface

const MAX_LINE = 5;
      MAX_WORD = 8;

type VOCABULARY =
     (again, all, and_, but, close, cut, drop, eat, examine, from, get,
      get_in_to, get_out_of, in_, inventory, it, lock, look, of_, off, on,
      oops, open, out, pick, put, quit, remove, score, take, time, to_,
      unlock, wait, wake, wear, with_,

      bars, couch, fuss_budget, hatch,

      return, rubbish,
      any_object, any_special_object, any_direction, any_word,

      go, north, east, south, west, up, down, enter, leave, sit, stand,

      sick_bay, hall_l, hall_m, hall_n, office, docking_port, scooter,
      store, room_, closed_hatch, unimplemented, wall,

      bracelet, cabinet, cage, case_of_fruit, computer, desk, hacksaw,
      isotopes, key, paste, plug, toolbox, uniform, watch,
      carrying, wearing);

     ACTOR = RECORD
       awake: BOOLEAN;
       room: VOCABULARY;
     end;

     APPETITE = (well_fed, fairly_hungry, very_hungry, awesomely_hungry, about_to_die);

     BAR_STATUS = (intact, one_cut, two_cut);

     DESCRIBE_TYPE = (brief, superbrief, verbose);

     SPOOLER_TYPE =
     (already_closed, already_open, already_standing, deep_space,
      dont_understand, get_up, no_exit, hatch_open, hatch_close,
      unimplemented_direction, which_direction);

     LOCATION = RECORD
       name:  STRING;
       line1: STRING;
       line2: STRING;
       line3: STRING;
       north: VOCABULARY;
       east:  VOCABULARY;
       south: VOCABULARY;
       west:  VOCABULARY;
       up:    VOCABULARY;
       down:  VOCABULARY;
       enter: VOCABULARY;
       leave: VOCABULARY;
       new:   BOOLEAN;
     end; (* location *)

     STATUS = (cuts, edible, eaten, heavy, wearable,
               lock_unlock, locked, unlocked, opened, closed, none);

     OBJECT_ = record
       name:           STRING;
       place:          VOCABULARY;
       characteristic: STATUS;
       lock:           STATUS;
       inside:         STATUS;
       new:            BOOLEAN;
       description:    STRING;
       line:           array [1..MAX_LINE] of STRING;
       eat_line:       array [1..2] of STRING;
      end; (* object *)

     WORD_TYPE = RECORD
       current, last: VOCABULARY;
       text: STRING;
     end;

const first_command = again;
      last_command  = with_;

      special_object_first = bars;
      special_object_last  = hatch;

      first_room = sick_bay;
      last_room  = store;

      object_first = bracelet;
      object_last  = watch;

function isCommand       (command_in:        VOCABULARY):BOOLEAN;
function isSpecialObject (special_object_in: VOCABULARY):BOOLEAN;
function isDirection     (direction_in:      VOCABULARY):BOOLEAN;
function isRoom          (room_in:           VOCABULARY):BOOLEAN;
function isObject        (object_in:         VOCABULARY):BOOLEAN;

implementation

function isCommand (command_in: VOCABULARY):BOOLEAN;
begin
  isCommand := command_in in [first_command..last_command];
end;

function isSpecialObject (special_object_in: VOCABULARY):BOOLEAN;
begin
  isSpecialObject := special_object_in in [bars..hatch];
end;

function isDirection (direction_in: VOCABULARY):BOOLEAN;
begin
  isDirection := direction_in in [north..down, enter, leave];
end;

function isRoom (room_in: VOCABULARY):BOOLEAN;
begin
  isRoom := room_in in [first_room..last_room];
end;

function isObject (object_in: VOCABULARY):BOOLEAN;
begin
  isObject := object_in in [object_first..object_last];
end;

end.
