unit general;

interface

uses crt, types, spooler;

type GENERAL_OBJECT = OBJECT (SPOOLER_OBJECT)
       points, clock, day, last_clock: INTEGER;
       is_sitting, is_wearing_watch, in_space: BOOLEAN;
       hungry: APPETITE;
       procedure init_general;
       procedure advance_time (chrons: INTEGER);
       procedure spool_time (text_in: STRING; chrons: INTEGER);
       procedure time_proc;
       procedure wait_proc;
       procedure score_proc;
       procedure status_line (room_name: STRING);
     end;

implementation

procedure GENERAL_OBJECT.init_general;
begin
  points := 0; clock := 4510; day := 1; last_clock := 4510;
  is_sitting := TRUE;
  is_wearing_watch := TRUE;
  in_space := TRUE;
  hungry := well_fed;
end;

procedure GENERAL_OBJECT.advance_time (chrons: INTEGER);
begin

  clock := clock + chrons;

  if in_space and (clock >= 4550) then
  begin

    spool;
    write ('   The scooter glides into the station''s docking port. The retro-thrusters'); spool;
    write ('bring the scooter to a halt. As the docking port fills with air the'); spool;
    write ('scooter''s hatch opens.'); spool;

    in_space := FALSE;
    room [scooter].leave := docking_port;
  end;

  if (last_clock <= 4690) and (clock >= 4690) then
  begin
    hungry := fairly_hungry;
    spool_text ('You are beginning to get hungry.');
  end;

  last_clock := clock;

end; (* advance_time *)

procedure GENERAL_OBJECT.spool_time (text_in: STRING; chrons: INTEGER);
begin
  spool_text (text_in);
  advance_time (chrons);
end;

procedure GENERAL_OBJECT.time_proc;
begin

  if is_wearing_watch then
  begin
    write ('Adjusted Galactic Standard Time (adjusted to your local day-cycle) is ', clock, '.');
    spool;
  end
  else
  begin
    spool_text ('Current Galactic Standard Time is impossible to determine, since you''ve');
    spool_text ('removed your chronometer.');
  end;

end; (* time_proc *)

procedure GENERAL_OBJECT.wait_proc;
begin

  spool_text ('Time passes...');
  advance_time (33);

end; (* wait_proc *)

procedure GENERAL_OBJECT.score_proc;
var line4: STRING;
begin

  if is_wearing_watch then
  begin
    write ('You''re currently getting brief descriptions. It is Day ',day,' of your adventure.'); spool;
    write ('Current Galactic Standard Time (adjusted to your local day cycle) is ',clock,'.'); spool;
    line4 := 'Your score is '+write_int(points)+' (out of 80 points), giving you the rank of ';
  end
  else
  begin
    write ('You''re currently getting brief descriptions. It is Day ',day,' of your adventure.'); spool;
    write ('Current Galactic Standard Time is impossible to determine, since you''ve'); spool;
    write ('removed your chronometer. Your score is ',points,' (out of 80 points), giving you the'); spool;
    line4 := 'rank of ';
  end;

  case points of
     0.. 4: line4 := line4 + 'Insignificant Nobody.';
     5..15: line4 := line4 + 'Rising Young Insignificant Nobody.';
    16..25: line4 := line4 + 'One-Day Flash on the Evening News.';
    26..39: line4 := line4 + 'Footnote in History.';
    40..49: line4 := line4 + 'International VIP.';
    50..59: line4 := line4 + 'Interplanetary Star.';
    60..79: line4 := line4 + 'Interstellar Superstar.';
        80: line4 := line4 + 'Intergalactic Mega-Hero.';
  end; (* case *)

  send_line  (line4);
  spool_text (line4);

end; (* score_proc *)

procedure GENERAL_OBJECT.status_line (room_name: STRING);
begin

  gotoxy (1,1); textcolor (black); textbackground (white);

  write ('                                                   Score:         Moves:        ');
  gotoxy (2,1); write (room_name);
  if is_sitting then write (', in the pilot''s couch');
  gotoxy (59,1); write (points);
  gotoxy (74,1); if is_wearing_watch then write (clock) else write ('0   ');

  gotoxy (1,25); textcolor (white); textbackground (black);

end; (* status_line *)

end. { screen }