unit obj_drop;

interface

uses types, actors;

type DROP_OBJECT = OBJECT (ACTORS_OBJECT)
       procedure drop_router;
       procedure core_drop (object_in, container: VOCABULARY; isSingleDrop: BOOLEAN);
       procedure drop_x_proc;
       procedure not_holding (object_in: VOCABULARY);
       procedure drop_proc   (object_in: VOCABULARY);
       procedure put_in_proc (object_in, container: VOCABULARY);
       procedure drop_all_proc (not1, not2: VOCABULARY);
       procedure put_all_in_proc (container, not1, not2: VOCABULARY);
     end;

implementation

procedure DROP_OBJECT.drop_router;
begin

  if phrase1 (drop) then
    drop_x_proc
  else if phrase2 (drop, scooter) then
    if (current_room in [scooter, docking_port]) then
      spool_text ('You''re not holding the scooter.')
    else
      spool_text ('You can''t see that here!')
  else if phrase2 (drop, any_object) then
    drop_proc (word[2].current)
  else if phrase2 (drop, all) then
    drop_all_proc (return, return)
  else if phrase4 (drop, all, but, any_object) then
    drop_all_proc (word[4].current, return)
  else if phrase6 (drop, all, but, any_object, and_, any_object) then
    drop_all_proc (word[4].current, word[6].current)

  else if phrase4 (put, any_object, in_, any_object) then
    put_in_proc (word[2].current, word[4].current)
  else if phrase4 (put, all, in_, any_object) then
    put_all_in_proc (word[4].current, return, return)
  else if phrase6 (put, all, in_, any_object, but, any_object) then
    put_all_in_proc (word[4].current, word[6].current, return)
  else if phrase8 (put, all, in_, any_object, but, any_object, and_, any_object) then
    put_all_in_proc (word[4].current, word[6].current, word[8].current)

end; (* drop_router *)

procedure DROP_OBJECT.core_drop (object_in, container: VOCABULARY; isSingleDrop: BOOLEAN);
begin

  if not isSingleDrop then
    write (list[object_in].name+': ');

  if isWearing (object_in) then
    spool_text ('You''ll have to remove it first.')
  else if is_sitting then
    spool_text ('You''ll have to leave the pilot''s couch first.')
  else
  begin
    if (container = return) then
      list[object_in].place := current_room
    else
      list[object_in].place := container;
    spool_time ('Dropped.', 7);
    if isSingleDrop then
    begin
      last_object := object_in; when := 1;
    end;
  end

end; (* core_drop *)

procedure DROP_OBJECT.drop_x_proc;
begin

  if no_objects_held then
    spool_text ('There''s nothing to drop!')
  else if one_object_held then
  begin
    spool_name ('[the ',object_held,']');
    core_drop (object_held, return, TRUE)
  end
  else
  begin
    spool_text ('[what do you want to drop?]');
    repeatCommand := drop;
  end;

end; (* drop_x_proc *)

procedure DROP_OBJECT.not_holding (object_in: VOCABULARY);
begin

  if isHere (object_in) then
    spool_name ('You''re not holding the ',object_in,'.')
  else
    spool_text ('You can''t see that here!');

end; (* not_holding *)

procedure DROP_OBJECT.drop_proc (object_in: VOCABULARY);
begin

  if isHeld (object_in) then
    core_drop (object_in, return, TRUE)
  else
    not_holding (object_in);

end; (* drop_proc *)

procedure DROP_OBJECT.put_in_proc (object_in, container: VOCABULARY);
begin

  if not isHeld (object_in) then
    spool_name ('You''re not holding the ',object_in,'.')
  else if is_sitting and isInRoom(container) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if isHere (container) then
  begin
    if list[container].inside = none then
      spool_name ('The ',container,' can''t hold anything!')
    else if list[container].inside = closed then
      spool_name ('You''ll have to open the ',container,' first.')
    else
      core_drop (object_in, container, TRUE);
  end
  else
    spool_name ('The ',container,' isn''t here!')

end; (* put_in_proc *)

procedure DROP_OBJECT.drop_all_proc (not1, not2: VOCABULARY);
var element: VOCABULARY;
    objects_held: INTEGER;
begin

  objects_held := 0;

  for element := object_first to object_last do
    if isHeld (element) and
       (element <> not1) and (element <> not2) then
    begin
      objects_held := objects_held + 1;
      core_drop (element, return, FALSE);
    end;

  if objects_held = 0 then
    spool_text ('There is nothing to drop!');

  if not isHeld (not1) and (not1 <> return) then
    not_holding (not1);

  if not isHeld (not2) and (not2 <> return) then
    not_holding (not2)

end; (* drop_all_proc *)

procedure DROP_OBJECT.put_all_in_proc (container, not1, not2: VOCABULARY);
var element: VOCABULARY;
    objects_held: INTEGER;
begin

  if not isHere (container) then
    spool_name ('The ',container,' isn''t here!')
  else if is_sitting and isInRoom(container) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if isHere (container) then
    if list[container].inside = none then
      spool_name ('The ',container,' can''t hold anything!')
    else if list[container].inside = closed then
      spool_name ('You''ll have to open the ',container,' first.')
    else
    begin
      objects_held := 0;

      for element := object_first to object_last do
        if isHeld (element) and
        (element <> not1) and (element <> not2) then
        begin
          objects_held := objects_held + 1;
          core_drop (element, container, FALSE);
        end;

      if objects_held = 0 then
        spool_text ('There is nothing to drop!');
    end;

  if isHeld (not1) and (not1 <> return) then
    not_holding (not1);

  if isHeld (not2) and (not2 <> return) then
    not_holding (not2);

end; (* put_all_in_proc *)

end.