unit obj_lock;

interface

uses types, obj_wear;

TYPE LOCK_OBJECT = OBJECT (WEAR_OBJECT)
       procedure open_proc  (object_in: VOCABULARY);
       procedure close_proc (object_in: VOCABULARY);
       procedure lock_proc (object_in: VOCABULARY);
       procedure lock_with_proc (object_in, key_object: VOCABULARY);
       procedure unlock_proc (object_in: VOCABULARY);
       procedure unlock_with_proc (object_in, key_object: VOCABULARY);
     end;

implementation

procedure LOCK_OBJECT.open_proc (object_in: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if is_sitting and not isHeld (object_in) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if (list[object_in].lock = locked) then
    spool_text ('It''s locked.')
  else if (list[object_in].inside = none) then
    spool_text ('You can''t open that!')
  else if (list[object_in].inside = opened) then
    spool_text ('It''s already open!')
  else
  begin
    list[object_in].inside := opened;
    current_line := 'Opening the '+list[object_in].name+' reveals ';
    list_objects_sub (object_in);
  end;

end; (* open_proc *)

procedure LOCK_OBJECT.close_proc (object_in: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if is_sitting and not isHeld (object_in) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if (list[object_in].inside = none) then
    spool_text ('You can''t close that!')
  else if (list[object_in].inside = closed) then
    spool_text ('It''s already closed!')
  else
  begin
    list[object_in].inside := closed;
    spool_name ('The ',object_in,' is now closed.');
  end;

end; (* close_proc *)

procedure LOCK_OBJECT.lock_proc (object_in: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if (list [object_in].lock = unlocked) then
  begin
    spool_name ('[what do you want to lock the ',object_in,' with?]');
    repeatCommand := lock;
  end
  else if (list [object_in].lock = locked) then
    spool_text ('It''s already locked!')
  else
    spool_text ('You can''t lock that!')

end; (* lock_proc *)

procedure LOCK_OBJECT.lock_with_proc (object_in, key_object: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if not isHeld (key_object) then
    spool_name ('You don''t have the ',key_object,'!')
  else if (object_in = key_object) then
    spool_text ('You can''t lock it with itself!')
  else if (list [object_in].lock = unlocked) then
    if (list [key_object].characteristic = lock_unlock) then
    begin
      list [object_in].lock := locked;
      spool_name ('The ',object_in,' is now locked.');
    end
    else
      spool_text ('That has no effect!')
  else if (list [object_in].lock = locked) then
    spool_text ('It''s already locked!')
  else
    spool_text ('You can''t lock that!')

end; (* lock_proc *)

procedure LOCK_OBJECT.unlock_proc (object_in: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if (list [object_in].lock = locked) then
  begin
    spool_name ('[what do you want to unlock the ',object_in,' with?]');
    repeatCommand := unlock;
  end
  else if (list [object_in].lock = unlocked) then
    spool_text ('It''s already unlocked!')
  else
    spool_text ('You can''t unlock that!')

end; (* unlock_proc *)

procedure LOCK_OBJECT.unlock_with_proc (object_in, key_object: VOCABULARY);
begin

  if not isHere (object_in) then
    spool_name ('You can''t see the ',object_in,' here!')
  else if not isHeld (key_object) then
    spool_name ('You don''t have the ',key_object,'!')
  else if (object_in = key_object) then
    spool_text ('You can''t unlock it with itself!')
  else if (list [object_in].lock = locked) then
    if (list [key_object].characteristic = lock_unlock) then
      if (object_in = cage) then
      begin
        spool_text ('This seems to be one of those new sophisticated all-purpose-key-proof');
        spool_text ('locks.');
      end
      else
      begin
        list [object_in].lock := unlocked;
        spool_name ('The ',object_in,' is now unlocked.');
      end
    else
      spool_text ('That has no effect!')
  else if (list [object_in].lock = unlocked) then
    spool_text ('It''s already unlocked!')
  else
    spool_text ('You can''t unlock that!')

end; (* unlock_proc *)

end.