unit obj_take;

interface

uses types, obj_drop;

type TAKE_OBJECT = OBJECT (DROP_OBJECT)
       procedure take_router;
       procedure core_take (object_in: VOCABULARY; isSingleTake: BOOLEAN);
       procedure take_x_proc;
       procedure take_proc (object_in: VOCABULARY);
       procedure take_from_proc (object_in, container: VOCABULARY);
       procedure take_all_proc (not1, not2: VOCABULARY);
       procedure take_all_from_proc (container, not1, not2: VOCABULARY);
     end;

implementation

procedure TAKE_OBJECT.take_router;
begin

  if phrase1 (take) then
    take_x_proc
  else if phrase2 (take, scooter) then
    if (current_room in [scooter, docking_port]) then
      spool_time ('A valiant attempt.', 7)
    else
      spool_text ('You can''t see any scooter here!')
  else if phrase2 (take, any_object) then
    take_proc (word[2].current)
  else if phrase4 (take, any_object, from, any_object) then
    take_from_proc (word[2].current, word[4].current)
  else if phrase2 (take, all) then
    take_all_proc (return, return)
  else if phrase4 (take, all, but, any_object) then
    take_all_proc (word[4].current, return)
  else if phrase6 (take, all, but, any_object, and_, any_object) then
    take_all_proc (word[4].current, word[6].current)
  else if phrase4 (take, all, from, any_object) then
    take_all_from_proc (word[4].current, return, return)
  else if phrase6 (take, all, from, any_object, but, any_object) then
    take_all_from_proc (word[4].current, word[6].current, return)
  else if phrase8 (take, all, from, any_object, but, any_object, and_, any_object) then
    take_all_from_proc (word[4].current, word[6].current, word[8].current)
  else
    spool_text ('I don''t understand that sentence.')

end; (* take_router *)

procedure TAKE_OBJECT.core_take (object_in: VOCABULARY; isSingleTake: BOOLEAN);
begin

  if not isSingleTake then
    write (list[object_in].name+': ');

  if is_sitting then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if isHeavy(object_in) then
    spool_text ('That is too heavy to carry.')
  else
  begin
    list[object_in].place := carrying;
    list[object_in].new := false;
    spool_time ('Taken.', 7);
    if isSingleTake then
    begin
      last_object := object_in; when := 1;
    end;
  end

end; (* core_take *)

procedure TAKE_OBJECT.take_x_proc;
begin

  if no_objects(current_room) then
    spool_text ('There''s nothing to take!')
  else if one_object(current_room) then
  begin
    spool_name ('[the ',sole_object(current_room),']');
    core_take (sole_object(current_room), TRUE)
  end
  else
  begin
    spool_text ('[what do you want to take?]');
    repeatCommand := take;
  end;

end; (* take_x_proc *)

procedure TAKE_OBJECT.take_proc (object_in: VOCABULARY);
begin

  if isHeld(object_in) then
    spool_text ('You''ve already got that.')
  else if isInRoom(object_in) then
    core_take (object_in, TRUE)
  else
    spool_text ('You can''t see that here.');

end; (* take_proc *)

procedure TAKE_OBJECT.take_from_proc (object_in, container: VOCABULARY);
begin

  if isHeld (object_in) then
    spool_name ('You''ve already got the ',object_in,'.')
  else if is_sitting and isInRoom(container) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if isHere (container) then
  begin
    if list[container].inside = none then
      spool_name ('The ',container,' can''t hold anything!')
    else if list[container].inside = closed then
      spool_name ('You''ll have to open the ',container,' first.')
    else
      if isHere (object_in) or (list[object_in].place = container) then
        core_take (object_in, TRUE)
      else
        spool_text ('You can''t see that here.');
  end
  else
    spool_name ('The ',container,' isn''t here!')

end; (* take_from_proc *)

procedure TAKE_OBJECT.take_all_proc (not1, not2: VOCABULARY);
var element: VOCABULARY;
    objects_in_room: INTEGER;
begin

  objects_in_room := 0;

  for element := object_first to object_last do
    if isInRoom(element) and
       (element <> not1) and (element <> not2) then
    begin
      objects_in_room := objects_in_room + 1;
      core_take (element, FALSE);
    end;

  if objects_in_room = 0 then
    spool_text ('There''s nothing to take!');

  if isHeld (not1) and (not1 <> return) then
    spool_name ('You already have the ',not1,'.');

  if isHeld (not2) and (not2 <> return) then
    spool_name ('You already have the ',not2,'.');

end; (* take_all_proc *)

procedure TAKE_OBJECT.take_all_from_proc (container, not1, not2: VOCABULARY);
var element: VOCABULARY;
    objects_in_container: INTEGER;
begin

  if not isHere (container) then
    spool_name ('The ',container,' isn''t here!')
  else if is_sitting and isInRoom(container) then
    spool_text ('You can''t reach it from the pilot''s couch.')
  else if isHere (container) then
    if list[container].inside = none then
      spool_name ('The ',container,' can''t hold anything!')
    else if list[container].inside = closed then
      spool_name ('You''ll have to open the ',container,' first.')
    else
    begin
      objects_in_container := 0;

      for element := object_first to object_last do
        if (list[element].place = container) and
        (element <> not1) and (element <> not2) then
        begin
          objects_in_container := objects_in_container + 1;
          core_take (element, FALSE);
        end;

      if objects_in_container = 0 then
        spool_text ('There''s nothing to take!');
    end;

  if isHeld (not1) and (not1 <> return) then
    spool_name ('You already have the ',not1,'.');

  if isHeld (not2) and (not2 <> return) then
    spool_name ('You already have the ',not2,'.');

end; (* take_all_from_proc *)

end.